@@ -95,7 +95,7 @@ def drop_all():
         _METADATA.drop_all(ENGINE)
 
 
-def get_job(job_id, limit=None):
+def get_job(job_id, limit=None, use_aps_id=False):
     """Return the job with the given job_id as a dict.
 
     The dict also includes any metadata or logs associated with the job.
@@ -154,8 +154,14 @@ def get_job(job_id, limit=None):
     if job_id:
         job_id = str(job_id)
 
-    result = ENGINE.execute(
-        JOBS_TABLE.select().where(JOBS_TABLE.c.job_id == job_id)).first()
+    if use_aps_id:
+        result = ENGINE.execute(
+            JOBS_TABLE.select().where(JOBS_TABLE.c.aps_job_id == job_id)
+        ).first()
+    else:
+        result = ENGINE.execute(
+            JOBS_TABLE.select().where(JOBS_TABLE.c.job_id == job_id)
+        ).first()
 
     if not result:
         return None
@@ -166,21 +172,22 @@ def get_job(job_id, limit=None):
         value = getattr(result, field)
         if value is None:
             result_dict[field] = value
-        elif field in ('sent_data', 'data', 'error'):
+        elif field in ("sent_data", "data", "error"):
             result_dict[field] = json.loads(value)
         elif isinstance(value, datetime.datetime):
             result_dict[field] = value.isoformat()
         else:
             result_dict[field] = str(value)
 
-    result_dict['metadata'] = _get_metadata(job_id)
-    result_dict['logs'] = _get_logs(job_id, limit=limit)
+    result_dict["metadata"] = _get_metadata(job_id)
+    result_dict["logs"] = _get_logs(job_id, limit=limit)
 
     return result_dict
 
 
-def add_pending_job(job_id, job_key, job_type, api_key,
-                    data=None, metadata=None, result_url=None):
+def add_pending_job(
+    job_id, job_key, job_type, api_key, data=None, metadata=None, result_url=None
+):
     """Add a new job with status "pending" to the jobs table.
 
     All code that adds jobs to the jobs table should go through this function.
@@ -242,24 +249,27 @@ def add_pending_job(job_id, job_key, job_type, api_key,
     conn = ENGINE.connect()
     trans = conn.begin()
     try:
-        conn.execute(JOBS_TABLE.insert().values(
-            job_id=job_id,
-            job_type=job_type,
-            status='pending',
-            requested_timestamp=datetime.datetime.now(),
-            sent_data=data,
-            result_url=result_url,
-            api_key=api_key,
-            job_key=job_key))
+        conn.execute(
+            JOBS_TABLE.insert().values(
+                job_id=job_id,
+                job_type=job_type,
+                status="pending",
+                requested_timestamp=datetime.datetime.now(),
+                sent_data=data,
+                result_url=result_url,
+                api_key=api_key,
+                job_key=job_key,
+            )
+        )
 
         # Insert any (key, value) metadata pairs that the job has into the
         # metadata table.
         inserts = []
         for key, value in list(metadata.items()):
-            type_ = 'string'
+            type_ = "string"
             if not isinstance(value, str):
                 value = json.dumps(value)
-                type_ = 'json'
+                type_ = "json"
 
             # Turn strings into unicode to stop SQLAlchemy
             # "Unicode type received non-unicode bind param value" warnings.
@@ -267,10 +277,7 @@ def add_pending_job(job_id, job_key, job_type, api_key,
             value = str(value)
 
             inserts.append(
-                {"job_id": job_id,
-                 "key": key,
-                 "value": value,
-                 "type": type_}
+                {"job_id": job_id, "key": key, "value": value, "type": type_}
             )
         if inserts:
             conn.execute(METADATA_TABLE.insert(), inserts)
@@ -319,11 +326,11 @@ def _validate_error(error):
             if isinstance(message, str):
                 return error
             else:
-                raise InvalidErrorObjectError(
-                    "error['message'] must be a string")
+                raise InvalidErrorObjectError("error['message'] must be a string")
         except (TypeError, KeyError):
             raise InvalidErrorObjectError(
-                "error must be either a string or a dict with a message key")
+                "error must be either a string or a dict with a message key"
+            )
 
 
 def _update_job(job_id, job_dict):
@@ -356,9 +363,8 @@ def _update_job(job_id, job_dict):
         job_dict["data"] = str(job_dict["data"])
 
     ENGINE.execute(
-        JOBS_TABLE.update()
-        .where(JOBS_TABLE.c.job_id == job_id)
-        .values(**job_dict))
+        JOBS_TABLE.update().where(JOBS_TABLE.c.job_id == job_id).values(**job_dict)
+    )
 
 
 def mark_job_as_completed(job_id, data=None):
@@ -426,8 +432,7 @@ def mark_job_as_failed_to_post_result(job_id):
 
     """
     update_dict = {
-        "error":
-            "Process completed but unable to post to result_url",
+        "error": "Process completed but unable to post to result_url",
     }
     _update_job(job_id, update_dict)
 
@@ -443,56 +448,70 @@ def delete_api_key(job_id):
     _update_job(job_id, {"api_key": None})
 
 
+def set_aps_job_id(job_id, aps_job_id):
+
+    _update_job(job_id, {"aps_job_id": aps_job_id})
+
+
 def _init_jobs_table():
     """Initialise the "jobs" table in the db."""
     _jobs_table = sqlalchemy.Table(
-        'jobs', _METADATA,
-        sqlalchemy.Column('job_id', sqlalchemy.UnicodeText, primary_key=True),
-        sqlalchemy.Column('job_type', sqlalchemy.UnicodeText),
-        sqlalchemy.Column('status', sqlalchemy.UnicodeText, index=True),
-        sqlalchemy.Column('data', sqlalchemy.UnicodeText),
-        sqlalchemy.Column('error', sqlalchemy.UnicodeText),
-        sqlalchemy.Column('requested_timestamp', sqlalchemy.DateTime),
-        sqlalchemy.Column('finished_timestamp', sqlalchemy.DateTime),
-        sqlalchemy.Column('sent_data', sqlalchemy.UnicodeText),
+        "jobs",
+        _METADATA,
+        sqlalchemy.Column("job_id", sqlalchemy.UnicodeText, primary_key=True),
+        sqlalchemy.Column("job_type", sqlalchemy.UnicodeText),
+        sqlalchemy.Column("status", sqlalchemy.UnicodeText, index=True),
+        sqlalchemy.Column("data", sqlalchemy.UnicodeText),
+        sqlalchemy.Column("error", sqlalchemy.UnicodeText),
+        sqlalchemy.Column("requested_timestamp", sqlalchemy.DateTime),
+        sqlalchemy.Column("finished_timestamp", sqlalchemy.DateTime),
+        sqlalchemy.Column("sent_data", sqlalchemy.UnicodeText),
+        sqlalchemy.Column("aps_job_id", sqlalchemy.UnicodeText),
         # Callback URL:
-        sqlalchemy.Column('result_url', sqlalchemy.UnicodeText),
+        sqlalchemy.Column("result_url", sqlalchemy.UnicodeText),
         # CKAN API key:
-        sqlalchemy.Column('api_key', sqlalchemy.UnicodeText),
+        sqlalchemy.Column("api_key", sqlalchemy.UnicodeText),
         # Key to administer job:
-        sqlalchemy.Column('job_key', sqlalchemy.UnicodeText),
-        )
+        sqlalchemy.Column("job_key", sqlalchemy.UnicodeText),
+    )
     return _jobs_table
 
 
 def _init_metadata_table():
     """Initialise the "metadata" table in the db."""
     _metadata_table = sqlalchemy.Table(
-        'metadata', _METADATA,
+        "metadata",
+        _METADATA,
         sqlalchemy.Column(
-            'job_id', sqlalchemy.ForeignKey("jobs.job_id", ondelete="CASCADE"),
-            nullable=False, primary_key=True),
-        sqlalchemy.Column('key', sqlalchemy.UnicodeText, primary_key=True),
-        sqlalchemy.Column('value', sqlalchemy.UnicodeText, index=True),
-        sqlalchemy.Column('type', sqlalchemy.UnicodeText),
-        )
+            "job_id",
+            sqlalchemy.ForeignKey("jobs.job_id", ondelete="CASCADE"),
+            nullable=False,
+            primary_key=True,
+        ),
+        sqlalchemy.Column("key", sqlalchemy.UnicodeText, primary_key=True),
+        sqlalchemy.Column("value", sqlalchemy.UnicodeText, index=True),
+        sqlalchemy.Column("type", sqlalchemy.UnicodeText),
+    )
     return _metadata_table
 
 
 def _init_logs_table():
     """Initialise the "logs" table in the db."""
     _logs_table = sqlalchemy.Table(
-        'logs', _METADATA,
+        "logs",
+        _METADATA,
         sqlalchemy.Column(
-            'job_id', sqlalchemy.ForeignKey("jobs.job_id", ondelete="CASCADE"),
-            nullable=False),
-        sqlalchemy.Column('timestamp', sqlalchemy.DateTime),
-        sqlalchemy.Column('message', sqlalchemy.UnicodeText),
-        sqlalchemy.Column('level', sqlalchemy.UnicodeText),
-        sqlalchemy.Column('module', sqlalchemy.UnicodeText),
-        sqlalchemy.Column('funcName', sqlalchemy.UnicodeText),
-        sqlalchemy.Column('lineno', sqlalchemy.Integer)
-        )
+            "job_id",
+            sqlalchemy.ForeignKey("jobs.job_id", ondelete="CASCADE"),
+            nullable=False,
+        ),
+        sqlalchemy.Column("timestamp", sqlalchemy.DateTime),
+        sqlalchemy.Column("message", sqlalchemy.UnicodeText),
+        sqlalchemy.Column("level", sqlalchemy.UnicodeText),
+        sqlalchemy.Column("module", sqlalchemy.UnicodeText),
+        sqlalchemy.Column("funcName", sqlalchemy.UnicodeText),
+        sqlalchemy.Column("lineno", sqlalchemy.Integer),
+    )
     return _logs_table
 
 
@@ -503,14 +522,14 @@ def _get_metadata(job_id):
     job_id = str(job_id)
 
     results = ENGINE.execute(
-        METADATA_TABLE.select().where(
-            METADATA_TABLE.c.job_id == job_id)).fetchall()
+        METADATA_TABLE.select().where(METADATA_TABLE.c.job_id == job_id)
+    ).fetchall()
     metadata = {}
     for row in results:
-        value = row['value']
-        if row['type'] == 'json':
+        value = row["value"]
+        if row["type"] == "json":
             value = json.loads(value)
-        metadata[row['key']] = value
+        metadata[row["key"]] = value
     return metadata
 
 
@@ -528,11 +547,15 @@ def _get_logs(job_id, limit=None):
 
     if not limit_is_valid:
         results = ENGINE.execute(
-            LOGS_TABLE.select().where(LOGS_TABLE.c.job_id == job_id)).fetchall()
+            LOGS_TABLE.select().where(LOGS_TABLE.c.job_id == job_id)
+        ).fetchall()
     else:
         results = ENGINE.execute(
-            LOGS_TABLE.select().where(LOGS_TABLE.c.job_id == job_id).\
-                order_by(LOGS_TABLE.c.timestamp.desc()).limit(limit)).fetchall()
+            LOGS_TABLE.select()
+            .where(LOGS_TABLE.c.job_id == job_id)
+            .order_by(LOGS_TABLE.c.timestamp.desc())
+            .limit(limit)
+        ).fetchall()
 
     results = [dict(result) for result in results]
 
@@ -542,11 +565,8 @@ def _get_logs(job_id, limit=None):
     return results
 
 
-def add_logs(job_id, message=None, level=None,
-                    module=None, funcName=None, lineno=None):
-    """for tests only
-
-    """
+def add_logs(job_id, message=None, level=None, module=None, funcName=None, lineno=None):
+    """for tests only"""
     if job_id:
         job_id = str(job_id)
     if message:
@@ -562,13 +582,16 @@ def add_logs(job_id, message=None, level=None,
     conn = ENGINE.connect()
     trans = conn.begin()
 
-    conn.execute(LOGS_TABLE.insert().values(
-        job_id=job_id,
-        timestamp=datetime.datetime.now(),
-        message=message,
-        level=level,
-        module=module,
-        funcName=funcName,
-        lineno=lineno))
+    conn.execute(
+        LOGS_TABLE.insert().values(
+            job_id=job_id,
+            timestamp=datetime.datetime.now(),
+            message=message,
+            level=level,
+            module=module,
+            funcName=funcName,
+            lineno=lineno,
+        )
+    )
     trans.commit()
     conn.close()
